"""
Flex

Name-US: Flex

Version: 1.0
Description-US: Hold Alt to have independent controls on each rectangle

"""

import c4d
import c4d.utils
from c4d import gui
import math





#These multi-line strings are going to be put into our rig's Python tag later
pythontagcode = """#FlexRectangle Tag Code""" + """
import c4d

def main():
    rec = op.GetObject()
    i = op[c4d.ID_USERDATA,1]
    obj = op[c4d.ID_USERDATA,2]
    rounding = op[c4d.ID_USERDATA,3]
    gap = op[c4d.ID_USERDATA,4]
    disap = op[c4d.ID_USERDATA,5]
    axis = op[c4d.ID_USERDATA,6]
    pp = obj.GetAllPoints() #Get All Point Positions
    p = obj.GetPolygon(i) #Get a specific polygon
    p_a = pp[p.a] #Position of the first point
    p_c = pp[p.c] #Position of the diagonal point
    rec[c4d.ID_BASEOBJECT_REL_POSITION] = c4d.utils.MixVec(p_a, p_c, 0.5) #Position of center point

    if axis == 2:
        width = abs(p_a[0]- p_c[0])-gap
        height = abs(p_a[1]- p_c[1])-gap
    elif axis == 1:
        width = abs(p_a[0]- p_c[0])-gap
        height = abs(p_a[2]- p_c[2])-gap
        rec[c4d.PRIM_PLANE] = 2
    else:
        width = abs(p_a[2]- p_c[2])-gap
        height = abs(p_a[1]- p_c[1])-gap
        rec[c4d.PRIM_PLANE] = 1

    rec[c4d.PRIM_RECTANGLE_WIDTH] = width
    rec[c4d.PRIM_RECTANGLE_HEIGHT] = height
    small = min((height*0.5),(width*0.5))
    if small > rounding:
        rec[c4d.PRIM_RECTANGLE_RADIUS] = rounding
    else:
        rec[c4d.PRIM_RECTANGLE_RADIUS] = small

    if small*2 < disap:
        rec[c4d.ID_BASEOBJECT_GENERATOR_FLAG] = False
    else:
        rec[c4d.ID_BASEOBJECT_GENERATOR_FLAG] = True

    """

pythontagcodeglobal = """#Dont Touch, Global Code""" + """
import c4d

def main():
    rec = op.GetObject()
    i = op[c4d.ID_USERDATA,1]
    obj = op[c4d.ID_USERDATA,2]
    rounding = obj[c4d.ID_USERDATA,1]
    gap = obj[c4d.ID_USERDATA,2]
    disap = obj[c4d.ID_USERDATA,3]
    axis = obj[c4d.ID_USERDATA,4]
    pp = obj.GetAllPoints() #Get All Point Positions
    p = obj.GetPolygon(i) #Get a specific polygon
    p_a = pp[p.a] #Position of the first point
    p_c = pp[p.c] #Position of the diagonal point
    rec[c4d.ID_BASEOBJECT_REL_POSITION] = c4d.utils.MixVec(p_a, p_c, 0.5) #Position of center point

    if axis == 2:
        width = abs(p_a[0]- p_c[0])-gap
        height = abs(p_a[1]- p_c[1])-gap
    elif axis == 1:
        width = abs(p_a[0]- p_c[0])-gap
        height = abs(p_a[2]- p_c[2])-gap
        rec[c4d.PRIM_PLANE] = 2
    else:
        width = abs(p_a[2]- p_c[2])-gap
        height = abs(p_a[1]- p_c[1])-gap
        rec[c4d.PRIM_PLANE] = 1

    rec[c4d.PRIM_RECTANGLE_WIDTH] = width
    rec[c4d.PRIM_RECTANGLE_HEIGHT] = height
    small = min((height*0.5),(width*0.5))
    if small > rounding:
        rec[c4d.PRIM_RECTANGLE_RADIUS] = rounding
    else:
        rec[c4d.PRIM_RECTANGLE_RADIUS] = small

    if small*2 < disap:
        rec[c4d.ID_BASEOBJECT_GENERATOR_FLAG] = False
    else:
        rec[c4d.ID_BASEOBJECT_GENERATOR_FLAG] = True
    """
def global_rig(object):
    doc.AddUndo(c4d.UNDOTYPE_CHANGE, object)

    #rounding value
    rounding = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    rounding[c4d.DESC_NAME] = "Rounding"
    rounding[c4d.DESC_UNIT] = c4d.DESC_UNIT_FLOAT
    rounding[c4d.DESC_STEP] = 1
    rounding[c4d.DESC_MIN] = 0
    rounding[c4d.DESC_MAX] = 1000
    rounding[c4d.DESC_FITH] = True
    object.AddUserData(rounding)

    #gap
    userdata_mix = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    userdata_mix[c4d.DESC_NAME] = "Gap"
    userdata_mix[c4d.DESC_UNIT] = c4d.DESC_UNIT_FLOAT
    userdata_mix[c4d.DESC_STEP] = 1
    userdata_mix[c4d.DESC_MIN] = -1000
    userdata_mix[c4d.DESC_MAX] = 1000
    userdata_mix[c4d.DESC_FITH] = True
    object.AddUserData(userdata_mix)

    #disappearing treshold
    disap = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    disap[c4d.DESC_NAME] = "Disappearing Treshold"
    disap[c4d.DESC_UNIT] = c4d.DESC_UNIT_FLOAT
    disap[c4d.DESC_STEP] = 1
    disap[c4d.DESC_MIN] = 0
    disap[c4d.DESC_MAX] = 1000
    disap[c4d.DESC_FITH] = True
    object.AddUserData(disap)

    #axis quicktab
    axis = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    axis[c4d.DESC_NAME] = "Axis"
    axis[c4d.DESC_CUSTOMGUI] = 200000281
    dropdown_values = c4d.BaseContainer()
    dropdown_values[0]="X"
    dropdown_values[1]="Y"
    dropdown_values[2]="Z"
    axis[c4d.DESC_CYCLE]=dropdown_values
    object.AddUserData(axis)

    object[c4d.ID_USERDATA,3] = 0
    object[c4d.ID_USERDATA,4] = 1

def global_link(object,id):
    doc.AddUndo(c4d.UNDOTYPE_CHANGE, object)

    #create a Python tag
    pythontag = c4d.BaseTag(c4d.Tpython)
    object.InsertTag(pythontag)
    doc.AddUndo(c4d.UNDOTYPE_NEW, pythontag)
    #set the Python code inside the tag
    pythontag[c4d.TPYTHON_CODE] = pythontagcodeglobal

    #polygon index
    ind = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    ind[c4d.DESC_NAME] = "Polygon Index"
    ind[c4d.DESC_UNIT] = c4d.DESC_UNIT_INT
    ind[c4d.DESC_MIN] = 0
    ind[c4d.DESC_MAX] = 50
    pythontag.AddUserData(ind)

    #parent link
    parent_plane = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    parent_plane[c4d.DESC_NAME] = "Parent Plane"
    pythontag.AddUserData(parent_plane)

    pythontag[c4d.ID_USERDATA,1] = id
    pythontag[c4d.ID_BASELIST_ICON_FILE] = "17995"


def new_rig(object,id):
    doc.AddUndo(c4d.UNDOTYPE_CHANGE, object)

    #create a Python tag
    pythontag = c4d.BaseTag(c4d.Tpython)
    object.InsertTag(pythontag)
    doc.AddUndo(c4d.UNDOTYPE_NEW, pythontag)
    #set the Python code inside the tag
    pythontag[c4d.TPYTHON_CODE] = pythontagcode

    #polygon index
    ind = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    ind[c4d.DESC_NAME] = "Polygon Index"
    ind[c4d.DESC_UNIT] = c4d.DESC_UNIT_INT
    ind[c4d.DESC_MIN] = 0
    ind[c4d.DESC_MAX] = 50
    pythontag.AddUserData(ind)

    #parent link
    parent_plane = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    parent_plane[c4d.DESC_NAME] = "Parent Plane"
    pythontag.AddUserData(parent_plane)

    #rounding value
    rounding = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    rounding[c4d.DESC_NAME] = "Rounding"
    rounding[c4d.DESC_UNIT] = c4d.DESC_UNIT_FLOAT
    rounding[c4d.DESC_STEP] = 1
    rounding[c4d.DESC_MIN] = 0
    rounding[c4d.DESC_MAX] = 1000
    rounding[c4d.DESC_FITH] = True
    pythontag.AddUserData(rounding)

    #gap
    userdata_mix = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    userdata_mix[c4d.DESC_NAME] = "Gap"
    userdata_mix[c4d.DESC_UNIT] = c4d.DESC_UNIT_FLOAT
    userdata_mix[c4d.DESC_STEP] = 1
    userdata_mix[c4d.DESC_MIN] = -1000
    userdata_mix[c4d.DESC_MAX] = 1000
    userdata_mix[c4d.DESC_FITH] = True
    pythontag.AddUserData(userdata_mix)

    #disappearing treshold
    disap = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    disap[c4d.DESC_NAME] = "Disappearing Treshold"
    disap[c4d.DESC_UNIT] = c4d.DESC_UNIT_FLOAT
    disap[c4d.DESC_STEP] = 1
    disap[c4d.DESC_MIN] = 0
    disap[c4d.DESC_MAX] = 1000
    disap[c4d.DESC_FITH] = True
    pythontag.AddUserData(disap)

    #axis quicktab
    axis = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    axis[c4d.DESC_NAME] = "Axis"
    axis[c4d.DESC_CUSTOMGUI] = 200000281
    dropdown_values = c4d.BaseContainer()
    dropdown_values[0]="X"
    dropdown_values[1]="Y"
    dropdown_values[2]="Z"
    axis[c4d.DESC_CYCLE]=dropdown_values
    pythontag.AddUserData(axis)

    pythontag[c4d.ID_BASELIST_NAME] = "Flex"
    pythontag[c4d.ID_BASELIST_ICON_FILE] = "465001216"
    pythontag[c4d.ID_USERDATA,1] = id
    pythontag[c4d.ID_USERDATA,4] = 0
    pythontag[c4d.ID_USERDATA,6] = 2

    return pythontag



def flex_independent():
    obj = doc.GetActiveObject()
    polycount = obj.GetPolygonCount() #Get number of polys
    pp = obj.GetAllPoints() #Get All Point Positions
    for i in range(polycount):
        rect  = c4d.BaseObject(5186) #create the rectangle
        doc.InsertObject(rect) #insert it
        name = polycount - i - 1 #create a new index from 0 to count
        rect[c4d.PRIM_RECTANGLE_ROUNDING] = True #enable the rounding checkbox
        rect[c4d.ID_BASELIST_NAME] = str(name) + "_Flex" #rename the rectangles
        new_rig(rect,i) #add the python tag to each rectangle
        tag = c4d.BaseObject.GetFirstTag(rect) #get the python tag
        tag[c4d.ID_USERDATA,2] = obj #insert the parent plane to each tag
    c4d.EventAdd()

def flex_global():
    obj = doc.GetActiveObject()
    polycount = obj.GetPolygonCount() #Get number of polys
    pp = obj.GetAllPoints() #Get All Point Positions
    global_rig(obj)
    for i in range(polycount):
        rect  = c4d.BaseObject(5186) #create the rectangle
        doc.InsertObject(rect) #insert it
        name = polycount - i - 1 #create a new index from 0 to count
        rect[c4d.PRIM_RECTANGLE_ROUNDING] = True #enable the rounding checkbox
        rect[c4d.ID_BASELIST_NAME] = str(name) + "_Flex" #rename the rectangles
        global_link(rect,i) #add the python tag to each rectangle
        tag = c4d.BaseObject.GetFirstTag(rect) #get the python tag
        tag[c4d.ID_USERDATA,2] = obj #insert the parent plane to each tag
    c4d.EventAdd()

# Main function
def main():
    bc = c4d.BaseContainer()
    if c4d.gui.GetInputState(c4d.BFM_INPUT_KEYBOARD,c4d.BFM_INPUT_CHANNEL,bc):
        if bc[c4d.BFM_INPUT_QUALIFIER] & c4d.QALT: # If ALT is pressed
            flex_independent()
        else:
            flex_global()

# Execute main()
if __name__=='__main__':
    main()